@@ -839,7 +839,7 @@ class EventCategorization(DataLoader):
     
     def scan_class_bounds(self, class_score_hist, n_bins:int, n_boundaries:int,
                           minimum_yields:Optional[Dict]=None,
-                          is_binary:bool=True) -> Dict:
+                          filter_regions:Optional[List[int]]=None) -> Dict:
         """
         Perform a boundary scan for the given MVA output class in the current channel
         
@@ -866,11 +866,8 @@ class EventCategorization(DataLoader):
             boundary_indices  = self.get_boundary_indices(n_bins, n_boundaries)
             n_combinations = len(boundary_indices)
             self.stdout.info(f"INFO: Total number of boundary combinations: {n_combinations}")
-            signal_yields     = self.get_region_yields(signal_hists, boundary_indices)
-            background_yields = self.get_region_yields(background_hists, boundary_indices)
-            if is_binary:
-                signal_yields     = signal_yields[:, 1:]
-                background_yields = background_yields[:, 1:]
+            signal_yields     = self.get_region_yields(signal_hists, boundary_indices, filter_regions=filter_regions)
+            background_yields = self.get_region_yields(background_hists, boundary_indices, filter_regions=filter_regions)
             # filter boundaries which the regions contain no signal or background yields
             valid_boundaries = np.all(background_yields != 0., axis=1) & np.all(signal_yields != 0., axis=1)
             # filter boundaries which the regions contain less than the minimum required yields for specific
@@ -880,12 +877,12 @@ class EventCategorization(DataLoader):
                     if sample not in all_hists:
                         raise RuntimeError(f"the sample {sample} is not used in testing")
                     min_yield = minimum_yields[sample]
-                    proc_yield = self.get_region_yields(all_hists[sample], boundary_indices)
-                    valid_boundaries &= np.all(proc_yield > min_yield, axis=1)
+                    sample_yields = self.get_region_yields(all_hists[sample], boundary_indices, filter_regions=filter_regions)
+                    valid_boundaries &= np.all(sample_yields > min_yield, axis=1)
             self.stdout.info(f"INFO: Number of valid boundary combinations: {valid_boundaries.sum()}")
             if valid_boundaries.sum() == 0:
                 self.stdout.info("INFO: Not enough combinations to scan due to minimum yield requirements. "
-                                 "No boundaries are set")
+                                 "No boundaries are set.")
                 return None
             combined_significances = get_combined_counting_significance(signal_yields[valid_boundaries],
                                                                         background_yields[valid_boundaries])
@@ -896,7 +893,7 @@ class EventCategorization(DataLoader):
             min_significance = combined_significances[min_boundary_idx]
             if ((max_significance - min_significance) / min_significance) < 0.1:
                 self.stdout.info("INFO: Improvement in Z is less than 10%. "
-                                 "No boundaries are set")
+                                 "No boundaries are set.")
                 return None
             max_bins         = boundary_indices[valid_boundaries][max_boundary_idx]
             max_boundary     = [bins[i] for i in max_bins]
@@ -944,6 +941,7 @@ class EventCategorization(DataLoader):
         """
         self.validate(check_df=True)
         channel = self.active_channel
+        channel_config  = self.config['channels'][channel]
         boundary_tree = MultiClassBoundaryTree(score_base_name=self.names["score"])
         
         # caching
@@ -956,16 +954,26 @@ class EventCategorization(DataLoader):
                 filename = self.get_file("boundary_data", channel=channel)
                 self.load_boundaries(filename)
                 return None
+            
+        exclude_categories = channel_config["exclude_categories"]
         
         self.stdout.info("INFO: Scanning score boundaries...")
         # run over class
         class_labels = self.get_class_labels()
         class_metadata = self.channel_metadata[channel]["classes"]
-        for class_label in class_labels:
+        for depth, class_label in enumerate(class_labels):
             class_config = class_metadata[class_label]['counting_significance']
             cut_maps = boundary_tree.get_cut_maps()
             # run over preceding class regions
             for branch_index, cut_expr in cut_maps.items():
+                if exclude_categories:
+                    if depth == 0:
+                        filter_regions = [category[depth] for category in exclude_categories]
+                    else:
+                        filter_regions = [category[depth] for category in exclude_categories \
+                                          if category[:depth] == list(branch_index)]
+                else:
+                    filter_regions = None
                 # multi-class case
                 if class_label:
                     if branch_index is not None:
@@ -973,15 +981,12 @@ class EventCategorization(DataLoader):
                         self.stdout.info(f"Class Name: {class_label} (region = {preceding_category_label})")
                     else:
                         self.stdout.info(f"Class Name: {class_label}")
-                    is_binary = False
-                else:
-                    is_binary = True
                 class_score_hist = self.get_class_score_hist(class_label, cut_expr)
                 result = self.scan_class_bounds(class_score_hist,
                                                 class_config['n_bins'],
                                                 class_config['n_boundaries'],
                                                 class_config['min_yield'],
-                                                is_binary=is_binary)
+                                                filter_regions=filter_regions)
                 if result is not None:
                     # append the new boundaries to the preceding branch
                     boundary_tree.set_boundaries(class_label, **result, branch_index=branch_index)
@@ -1753,7 +1758,8 @@ class EventCategorization(DataLoader):
         return a[mask]
 
     @staticmethod
-    def get_region_yields(data:np.ndarray, boundary_indices:np.ndarray):
+    def get_region_yields(data:np.ndarray, boundary_indices:np.ndarray,
+                          filter_regions:Optional[List[int]]=None):
         # much faster implementation
         n = data.shape[0]
         d1, d2 = boundary_indices.shape[0], boundary_indices.shape[1]
@@ -1761,4 +1767,7 @@ class EventCategorization(DataLoader):
         new_data = np.insert(data, n, 0)
         result = np.add.reduceat(new_data, slice_indices).reshape(d1, d2 + 2)
         result = np.delete(result, -1, axis=1)
+        # remove yields from disgarded region(s)
+        if filter_regions is not None:
+            result = np.delete(result, filter_regions, axis=1)
         return result
\ No newline at end of file
