@@ -86,7 +86,7 @@ class NTupleConversionTool(AnalysisBase):
             # does not support chunksize
             from quickstats.utils.data_conversion import root2dataframe
             df = root2dataframe(filename, treename, columns=columns,
-                                remove_non_standard_types=False,
+                                mode=1,
                                 downcast=downcast, library="root")
             df_iter = [df]
         elif library == "root_pandas":
