@@ -2,6 +2,7 @@ from typing import Optional, List, Dict, Union, Callable
 import os
 import glob
 import json
+import math
 
 import numpy as np
 
@@ -240,7 +241,55 @@ class NTupleProcessTool(ConfigurableObject):
                            outdir=outdir,
                            samples=samples,
                            subdirs=subdirs)
-            
+    
+    def _process_exported_data(self, df, filename:str=None):
+        import pandas as pd
+        if (filename is None) or (not os.path.exists(filename)):
+            return df, None
+        with open(filename, 'r') as file:
+            data = json.load(file)
+        cutflow_names = df['name'].values
+        yield_values = [data.pop(name, None) for name in cutflow_names]
+        if not data:
+            return df, yield_values
+        extra_cutflow = {}
+        extra_yields = {}
+        for key, value in data.items():
+            if key.startswith("CUTFLOW"):
+                key = key.strip("CUTFLOW").strip()
+                extra_cutflow[key] = int(value)
+            elif key.startswith("WEIGHT"):
+                key = key.strip("WEIGHT").strip()
+                extra_yields[key] = float(value)
+        n_events = df['all'].values[0]
+        rows = []
+        for key, value in extra_cutflow.items():
+            row = {"name": key, "all": None, "pass": value,
+                   "efficiency": None,
+                   "cumulative_efficiency": 100 * (value / n_events)}
+            rows.append(row)
+            yield_values.append(extra_yields.get(key, None))
+        df = pd.concat([df, pd.DataFrame(rows)], ignore_index=True)
+        return df, yield_values
+    
+    def _get_efficiency_values(self, df, weight_column:str):
+        weight_values = df[weight_column].values
+        prev_pass_values = df['all'].values
+        this_pass_values = df['pass'].values
+        efficiency = [100]
+        cumul_efficiency = [100]
+        for i in range(1, len(weight_values)):
+            cumul_efficiency.append(100 * (weight_values[i] / weight_values[0]))
+            prev_pass_value = prev_pass_values[i]
+            if (prev_pass_value is None) or math.isnan(prev_pass_value):
+                efficiency.append(None)
+                continue
+            last_idx = np.where(this_pass_values == prev_pass_value)
+            if len(last_idx[0]) == 0:
+                raise RuntimeError("failed to calculate efficiency")
+            efficiency.append(100 * (weight_values[i] / weight_values[last_idx][0]))
+        return efficiency, cumul_efficiency
+    
     def _merge_cutflow_data(self, fnames:List[str], outname:str):
         import pandas as pd
         merged_df = None
@@ -256,35 +305,28 @@ class NTupleProcessTool(ConfigurableObject):
                 weight_fname = None
             if not os.path.exists(cutflow_fname):
                 raise FileNotFoundError(f'missing cutflow file "{cutflow_fname}"')
-            cutflow_df = pd.read_csv(cutflow_fname).set_index(['name'])
-            if (weight_fname is None) or not os.path.exists(weight_fname):
-                weight_data = None
-            else:
-                with open(weight_fname, 'r') as file:
-                    weight_data = np.array(list(json.load(file).values()))
+            cutflow_df = pd.read_csv(cutflow_fname)
+            cutflow_df, yield_values = self._process_exported_data(cutflow_df, weight_fname)
             if merged_df is None:
                 merged_df = cutflow_df.copy()
-                if weight_data is not None:
-                    assert len(merged_df) == len(weight_data)
-                    merged_df['yield'] = weight_data
+                if yield_values is not None:
+                    merged_df['yield'] = yield_values
             else:
                 merged_df['all']  += cutflow_df['all']
                 merged_df['pass'] += cutflow_df['pass']
                 if 'yield' in merged_df:
-                    if weight_data is None:
+                    if yield_values is None:
                         raise RuntimeError(f'missing weight file that is in association with the '
                                            f'cutflow file "{cutflow_fname}"')
-                    merged_df['yield'] += weight_data
+                    merged_df['yield'] += yield_values
+        efficiency, cumul_efficiency = self._get_efficiency_values(merged_df, 'pass')
+        merged_df['efficiency'] = efficiency
+        merged_df['cumulative_efficiency'] = cumul_efficiency
         if 'yield' in merged_df:
-            yields = merged_df['yield'].values
-            yield_efficiency = [100]
-            yield_cumul_efficiency = [100]
-            for i in range(1, len(yields)):
-                yield_efficiency.append(100 * (yields[i] / yields[i-1]))
-                yield_cumul_efficiency.append(100 * (yields[i] / yields[0]))
+            yield_efficiency, yield_cumul_efficiency = self._get_efficiency_values(merged_df, 'yield')
             merged_df['yield_efficiency'] = yield_efficiency
             merged_df['yield_cumulative_efficiency'] = yield_cumul_efficiency
-        merged_df.to_csv(outname)
+        merged_df.to_csv(outname, index=False)
         self.stdout.info(f'INFO: Saved cutflow data as "{outname}"')
         
     def load_cutflow_report(self, samples:Optional[List[str]]=None):
@@ -311,7 +353,10 @@ class NTupleProcessTool(ConfigurableObject):
         self.cutflow_report = cutflow_report
 
     def plot_cutflow_report(self, samples:Optional[List[str]]=None,
-                            label_map:Optional[Dict]=None):
+                            label_map:Optional[Dict]=None,
+                            rotation:int=15,
+                            pad:float=3.0,
+                            figsize=(17,8)):
 
         import matplotlib.pyplot as plt
         
@@ -336,15 +381,15 @@ class NTupleProcessTool(ConfigurableObject):
                 print(f'WARNING: Missing cutflow report for the sample "{sample}". Skipped.')
                 continue
 
-            df = self.cutflow_report[sample] 
+            df = self.cutflow_report[sample].fillna(0)
 
             cut_labels = [label_map.get(name, name) for name in df["name"]]
 
-            fig, axs = plt.subplots(3, 1, figsize=(17, 8))
-            fig.tight_layout(pad=3.0)
+            fig, axs = plt.subplots(3, 1, figsize=figsize)
+            fig.tight_layout(pad=pad)
 
             for i in range(len(axs)): # set x-axes style
-                axs[i].tick_params(axis='x', rotation =15, labelsize=10) 
+                axs[i].tick_params(axis='x', rotation=rotation, labelsize=10) 
 
             # Yields bar chart 
             axs[0].bar(cut_labels, df["yield"], color=colors[0])
