@@ -41,17 +41,23 @@ from .core import DelimitedStr
 @click.option('--offset/--no-offset', default=True, show_default=True,
               help='Offset likelihood')
 @click.option('-v', '--verbosity',  default="INFO", show_default=True,
-              help='verbosity level ("DEBUG", "INFO", "WARNING", "ERROR")')
+              help='Verbosity level ("DEBUG", "INFO", "WARNING", "ERROR")')
+@click.option('--version', type=int,  default=2, show_default=True,
+              help='Version of XMLWSBuilder to use (Choose between 1 and 2).')
 def build_xml_ws(**kwargs):
     """
     Build workspace from XML config files
     """
+    version == kwargs.pop("version")
     _kwargs = {}
     for arg_name in ["source", "use_binned", "data_storage_type",
                      "basedir", "verbosity"]:
         _kwargs[arg_name] = kwargs.pop(arg_name)
     _kwargs['minimizer_config'] = kwargs
-    from quickstats.components.workspaces import XMLWSBuilder
+    if version == 1:
+        from quickstats.components.workspaces import XMLWSBuilderV1 as XMLWSBuilder
+    else:        
+        from quickstats.components.workspaces import XMLWSBuilder
     builder = XMLWSBuilder(**_kwargs)
     builder.generate_workspace()
     
@@ -62,6 +68,9 @@ def build_xml_ws(**kwargs):
               help='Override input workspace path from the xml/json file.')
 @click.option('--output_workspace', 
               help='Override output workspace path from the xml/json file.')
+@click.option('--import-class-code/--no-import-class-code', 'import_class_code',
+              default=True, show_default=True,
+              help='Import class code')
 @click.option('-t', '--minimizer_type', default="Minuit2", show_default=True,
               help='Minimizer type')
 @click.option('-a', '--minimizer_algo', default="Migrad", show_default=True,
@@ -96,13 +105,15 @@ def modify_ws(**kwargs):
     """
     infile = kwargs.pop("input_workspace")
     outfile = kwargs.pop("output_workspace")
+    import_class_code = kwargs.pop('import_class_code')
     _kwargs = {}
     for arg_name in ["source", "verbosity"]:
         _kwargs[arg_name] = kwargs.pop(arg_name)
     _kwargs['minimizer_config'] = kwargs
     from quickstats.components.workspaces import XMLWSModifier
     modifier = XMLWSModifier(**_kwargs)
-    modifier.create_modified_workspace(infile=infile, outfile=outfile)
+    modifier.create_modified_workspace(infile=infile, outfile=outfile,
+                                       import_class_code=import_class_code)
     
 kItemChoices = ['workspace', 'dataset', 'category', 'snapshot', 'pdf',
                 'function', 'poi', 'nuisance_parameter', 'global_observable',
@@ -201,6 +212,9 @@ def compare_ws(**kwargs):
               help='Optimize constant terms')
 @click.option('--offset/--no-offset', default=True, show_default=True,
               help='Offset likelihood')
+@click.option('--import-class-code/--no-import-class-code', 'import_class_code',
+              default=True, show_default=True,
+              help='Import class code')
 @click.option('-v', '--verbosity',  default="INFO", show_default=True,
               help='verbosity level ("DEBUG", "INFO", "WARNING", "ERROR")')
 def combine_ws(**kwargs):
@@ -209,6 +223,7 @@ def combine_ws(**kwargs):
     """
     infile = kwargs.pop("input_workspace")
     outfile = kwargs.pop("output_workspace")
+    import_class_code = kwargs.pop('import_class_code')
     if infile is not None:
         infiles = {}
         for item in infile.split(","):
@@ -229,5 +244,6 @@ def combine_ws(**kwargs):
     combiner.create_combined_workspace(infiles=infiles, outfile=outfile,
                                        save_rename_ws=save_rename_ws,
                                        save_combine_ws=save_combine_ws,
-                                       save_final_ws=True)    
+                                       save_final_ws=True,
+                                       import_class_code=import_class_code)    
     
\ No newline at end of file
