@@ -1,21 +1,24 @@
 from typing import Optional, Dict, List, Union
-from quickstats import GeneralEnum, DescriptiveEnum
-import enum
+import copy
 
+from quickstats import GeneralEnum, DescriptiveEnum
 from quickstats.components import AnalysisObject
 from quickstats.utils.common_utils import parse_config
 from quickstats.components.basics import WSArgument
 
-class AsimovType(enum.Enum):
-    Temp                   = -999
-    S_NP_Nom               = -2
-    B_NP_Nom               = -1
-    B_NP_Fit               = 0
-    S_NP_Fit               = 1
-    S_NP_Fit_muhat         = 2
-    B_unconstrained_NP_Fit = 3
-    S_unconstrained_NP_Fit = 4
-    S_unconstrained_NP_Fit_muhat = 5
+class AsimovType(DescriptiveEnum):
+    Temp                         = (-999, "do not generate any asimov")
+    S_NP_Nom                     = (-2, "S + B Asimov (mu = 1) with nominal NP values")
+    B_NP_Nom                     = (-1, "B only Asimov (mu = 0) with nominal NP values")
+    B_NP_Fit                     = (0, "B only Asimov (mu = 0) with NP values profiled to data (fit with mu fixed at 0)")
+    S_NP_Fit                     = (1, "S + B Asimov (mu = 1) with NP values profiled to data (fit with mu fixed at 1)")
+    S_NP_Fit_muhat               = (2, "S + B Asimov (mu = 1) with NP values profiled to data (fit with mu floating)")
+    B_unconstrained_NP_Fit       = (3, "B only Asimov (mu = 0) with unconstrained NP values profiled to data "
+                                   "(fit with mu fixed at 0 and constrained NP fixed at 0)")
+    S_unconstrained_NP_Fit       = (4, "S + B Asimov (mu = 1) with unconstrained NP values profiled to data "
+                                   "(fit with mu fixed at 1 and constrained NP fixed at 0)")
+    S_unconstrained_NP_Fit_muhat = (5, "S + B Asimov (mu = 1) with unconstrained NP values profiled to data "
+                                   "(fit with mu floating and constrained NP fixed at 0)")
     
 class AsimovGenerator(AnalysisObject):
 
@@ -134,14 +137,18 @@ class AsimovGenerator(AnalysisObject):
         }
         return self.model.generate_asimov(**kwargs)
         
-    def generate_standard_asimov(self, asimov_types:Optional[Union[List[AsimovType], List[int]]]=None,
+    def generate_standard_asimov(self, asimov_types:Optional[Union[List[AsimovType], List[int], List[str]]]=None,
                                  asimov_names:Optional[Union[List[str], str]]=None,
                                  asimov_snapshots:Optional[Union[List[str], str]]=None,
-                                 poi_scale:Optional[float]=None):
+                                 poi_scale:Optional[float]=None, do_import:bool=True):
+        single_asimov = False
         if asimov_types is None:
             asimov_types = self.DEFAULT_ASIMOV_TYPES
-        if isinstance(asimov_types, (int, AsimovType)):
-            asimov_types = [asimov_types]
+        if isinstance(asimov_types, (int, str, AsimovType)):
+            asimov_types = [AsimovType.parse(asimov_types)]
+            single_asimov = True
+        else:
+            asimov_types = [AsimovType.parse(atype) for atype in asimov_types]
         if asimov_names is not None:
             if isinstance(asimov_names, str):
                 asimov_names = [asimov_names]
@@ -154,11 +161,10 @@ class AsimovGenerator(AnalysisObject):
             poi_scale = 1.0
         poi_name = self.poi.GetName()
         self.save_snapshot("temp", WSArgument.MUTABLE)
+        asimov_datasets = []
         for i, asimov_type in enumerate(asimov_types):
             self.load_snapshot("temp")
-            if isinstance(asimov_type, int):
-                asimov_type = AsimovType(asimov_type)
-            kwargs = self.ASIMOV_SETTINGS[asimov_type]
+            kwargs = copy.deepcopy(self.ASIMOV_SETTINGS[asimov_type])
             if kwargs is None:
                 continue
             kwargs['poi_name'] = poi_name
@@ -169,6 +175,11 @@ class AsimovGenerator(AnalysisObject):
                 kwargs['asimov_name'] = asimov_names[i]
             if asimov_snapshots is not None:
                 kwargs['asimov_snapshot'] = asimov_snapshots[i]
-            kwargs['minimizer_options'] = self.minimizer_options
-            kwargs['nll_options'] = self.nll_commands
-            self.model.generate_asimov(**kwargs)
\ No newline at end of file
+            kwargs['minimizer_options'] = self.minimizer.config
+            kwargs['nll_options'] = list(self.minimizer.nll_commands.values())
+            kwargs['do_import'] = do_import
+            asimov_dataset = self.model.generate_asimov(**kwargs)
+            asimov_datasets.append(asimov_dataset)
+        if single_asimov:
+            return asimov_datasets[0]
+        return asimov_datasets
\ No newline at end of file
