@@ -20,7 +20,7 @@ class ExtendedMinimizer(AbstractObject):
     _DEFAULT_MINIMIZER_OPTION_ = {
         'minimizer_type': 'Minuit2',
         'minimizer_algo': 'Migrad',        
-        'opt_const': 2,
+        'optimize': 2,
         'verbose': 0,
         'save': 0,
         'timer': 0,
@@ -39,14 +39,16 @@ class ExtendedMinimizer(AbstractObject):
         'max_calls': -1,
         'max_iters': -1,
         'n_sigma': 1,
-        'precision': 1,
-        'default_strategy': 1,
+        'precision': 0.001,
+        'strategy': 1,
         'print_level': -1
     }
     
     _DEFAULT_NLL_OPTION_ = {
         'num_cpu': 1,
-        'offset': True
+        'offset': True,
+        'batch_mode': False,
+        'int_bin_precision': -1
     }
     
     @property
@@ -113,10 +115,10 @@ class ExtendedMinimizer(AbstractObject):
             
     @staticmethod
     def _configure_default_minimizer_options(minimizer_type='Minuit2', minimizer_algo='Migrad',
-                                             default_strategy=0, print_level=-1,
+                                             strategy=0, print_level=-1,
                                              debug_mode=False):
         ROOT.Math.MinimizerOptions.SetDefaultMinimizer(minimizer_type, minimizer_algo)
-        ROOT.Math.MinimizerOptions.SetDefaultStrategy(default_strategy)
+        ROOT.Math.MinimizerOptions.SetDefaultStrategy(strategy)
         ROOT.Math.MinimizerOptions.SetDefaultPrintLevel(print_level)
         if debug_mode:
             ROOT.Math.MinimizerOptions.SetDefaultPrintLevel(1)
@@ -130,7 +132,7 @@ class ExtendedMinimizer(AbstractObject):
     def configure_default_minimizer_options(self):
         self._configure_default_minimizer_options(self.config['minimizer_type'],
                                                   self.config['minimizer_algo'],
-                                                  self.config['default_strategy'],
+                                                  self.config['strategy'],
                                                   self.config['print_level'])
         return None
     
@@ -216,17 +218,21 @@ class ExtendedMinimizer(AbstractObject):
             self.minimizer = ROOT.RooMinimizer(self.nll)
             self.minimizer.setOffsetting(self.config['minimizer_offset'])
             self.minimizer.setPrintLevel(self.config['print_level'])
-            self.minimizer.optimizeConst(self.config['opt_const'])
+            self.minimizer.optimizeConst(self.config['optimize'])
             self.minimizer.setMinimizerType(self.config['minimizer_type'])
             self.minimizer.setEvalErrorWall(self.config['do_ee_wall'])
             self.minimizer.setPrintEvalErrors(self.config['num_ee'])
             self.minimizer.setVerbose(self.config['verbose'])
             self.minimizer.setProfile(self.config['timer'])
-            self.minimizer.setStrategy(self.config['default_strategy'])
+            self.minimizer.setStrategy(self.config['strategy'])
             self.minimizer.setEps(self.config['eps'])
-            
-            if self.config['max_calls'] != -1:
+            # https://root.cern/doc/master/VariableMetricBuilder_8cxx_source.html
+            if self.config['max_calls'] != -1: 
                 self.minimizer.setMaxFunctionCalls(self.config['max_calls'])
+            else:
+                max_calls = 5000 * self.pdf.getVariables().getSize()
+                self.minimizer.setMaxFunctionCalls(max_calls)
+                
             if self.config['max_iters'] != -1:
                 self.minimizer.setMaxIterations(self.config['max_iters'])
     
@@ -413,8 +419,8 @@ class ExtendedMinimizer(AbstractObject):
     
     def robust_minimize(self):
 
-        strategy = self.config['default_strategy']
-        retry = self.config['retry']
+        strategy = self.config['strategy']
+        retry    = self.config['retry']
         minimizer_type = self.config['minimizer_type']
         minimizer_algo = self.config['minimizer_algo']
 
@@ -423,26 +429,26 @@ class ExtendedMinimizer(AbstractObject):
             if (strategy < 2):
                 strategy += 1
             retry -= 1
-            self.stdout.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}. '
-                          'Retrying with strategy {}'.format(self.name, status, strategy))
+            self.stdout.error(f'ExtendedMinimizer::robust_minimize("{self.name}") fit failed with status {status}. '
+                              f'Retrying with strategy {strategy}')
             self.minimizer.setStrategy(strategy)
             status = self.minimizer.minimize(minimizer_type, minimizer_algo)
         
         if status not in [0, 1]:
-            self.stdout.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}'.format(self.name, status))
+            self.stdout.error(f'ExtendedMinimizer::robust_minimize("{self.name}") fit failed with status {status}')
         
-        self.minimizer.setStrategy(self.config['default_strategy'])
+        self.minimizer.setStrategy(self.config['strategy'])
         
         return status
     
     def use_limits(self, par:"ROOT.RooRealVar", val:float):
         if (val < par.getMin()):
-            self.stdout.warning('ExtendedMinimizer::use_limits("{}") {} = {} limited by minimum at {}'.format(
-                            self.name, par.GetName(), val, par.getMin()))
+            self.stdout.warning(f'ExtendedMinimizer::use_limits("{self.name}") {par.GetName()} = {val} '
+                                f'limited by minimum at {par.getMin()}')
             return par.getMin()
         elif (val > par.getMax()):
-            self.stdout.warning('ExtendedMinimizer::use_limits("{}") {} = {} limited by maximum at {}'.format(
-                            self.name, par.GetName(), val, par.getMax()))
+            self.stdout.warning(f'ExtendedMinimizer::use_limits("{self.name}") {par.GetName()} = {val} '
+                                f'limited by maximum at {par.getMax()}')
             return par.getMax()
         else:
             return val
@@ -478,7 +484,7 @@ class ExtendedMinimizer(AbstractObject):
             _hi = hi if not math.isnan(hi) else -1.0
             v.setAsymError(_lo, _hi)
             
-            self.stdout.info('ExtendedMinimizer::minimize("{}")'.format(self.name))
+            self.stdout.info(f'ExtendedMinimizer::minimize("{self.name}")')
             v.Print()
 
     # _____________________________________________________________________________
@@ -503,7 +509,7 @@ class ExtendedMinimizer(AbstractObject):
         damping_factor = 1.0
         guess_to_corr = {}
         t_mu = ROOT.TMath.QuietNaN()
-        front_str = 'ExtendedMinimizer::findSigma("{}") '.format(self.name)
+        front_str = f'ExtendedMinimizer::findSigma("{self.name}") '
         
         
         if precision <= 0.0:
@@ -515,9 +521,8 @@ class ExtendedMinimizer(AbstractObject):
         temp_options = {**scan_options}
         snapshot = self.cond_set.snapshot()
         for i in range(max_iter):
-            self.stdout.info('{} Parameter {} {}sigma '
-                         'iteration {}: start {} (MLE{})'.format(front_str, par.GetName(),
-                         n_sigma, i+1, val_guess, val_guess-val_mle))
+            self.stdout.info(f'{front_str} Parameter {par.GetName()} {n_sigma}sigma '
+                             f'iteration {i + 1}: start {val_guess} (MLE{val_guess - val_mle})')
             val_pre = val_guess
             
             poi_set = ROOT.RooArgSet(par).snapshot()
@@ -548,7 +553,7 @@ class ExtendedMinimizer(AbstractObject):
             for guess in guess_to_corr:
                 if (abs(guess - val_pre) < direction*val_pre*0.02):
                     damping_factor *= 0.8
-                    self.stdout.info('{} Changing damping factor to {}'.format(front_str, damping_factor))
+                    self.stdout.info(f'{front_str} Changing damping factor to {damping_factor}')
                     n_damping += 1
                     if n_damping > 10:
                         n_damping = 1
@@ -565,9 +570,9 @@ class ExtendedMinimizer(AbstractObject):
             
             
             self.stdout.info('{} {} {:.3f} (MLE {:.3f}) -> {:.3f} (MLE {:.3f}), '
-                          'change {:.3f}, precision {:.3f}, -2lnL {:.4f}, sigma(guess) {:.3f})'.format(
-                          front_str, param_name, val_pre, val_pre - val_mle, val_guess, val_guess - val_mle,
-                          delta, rel_precision, t_mu, sigma_guess))
+                             'change {:.3f}, precision {:.3f}, -2lnL {:.4f}, sigma(guess) {:.3f})'.format(
+                             front_str, param_name, val_pre, val_pre - val_mle, val_guess, val_guess - val_mle,
+                             delta, rel_precision, t_mu, sigma_guess))
             self.stdout.info('{} NLL:                 {}'.format(front_str, nll))
             self.stdout.info('{} delta(NLL):          {}'.format(front_str, nll - nll_min))
             self.stdout.info('{} nsigma*sigma(pre):   {}'.format(front_str, abs(val_pre - val_mle)))
@@ -582,12 +587,12 @@ class ExtendedMinimizer(AbstractObject):
             if abs(delta) <= rel_precision:
                 break
         if i >= max_iter:
-            self.stdout.error('ERROR: find_sigma failed after {} iterations'.format(i+1))
+            self.stdout.error(f'ERROR: find_sigma failed after {i + 1} iterations')
             return ROOT.TMath.QuietNan()
         
         err = val_guess - val_mle
         self.stdout.info('{} {} {}sigma = {:.3F} at -2lnL = {:4f} after {} iterations'.format(
-                     front_str, par.GetName(), n_sigma, err, t_mu, i+1))
+                         front_str, par.GetName(), n_sigma, err, t_mu, i+1))
         
         return err
     
