@@ -71,6 +71,7 @@ class ExtendedModel(AbstractObject):
         self.fix_cache = fix_cache
         self.fix_multi = fix_multi
         self.interpolation_code = interpolation_code
+        self.last_fit_status = None
         if minimizer_cls is None:
             self.minimizer_cls = ExtendedMinimizer
         else:
@@ -269,7 +270,7 @@ class ExtendedModel(AbstractObject):
         if self.data_name is None:
             data_names = [i.GetName() for i in ws.allData()]
             if not data_names:
-                raise RuntimeError(f"no datasets found in the workspace: {ws_name}")
+                raise RuntimeError(f"no datasets found in the workspace: {ws.GetName()}")
             self.data_name = data_names[0]
         data = ws.data(self.data_name)
         # in case there is a bug in hash map
@@ -1003,10 +1004,10 @@ class ExtendedModel(AbstractObject):
         mutable_vars = self.get_variables(WSArgument.MUTABLE)
         ws.saveSnapshot(nom_vars_name, mutable_vars)
         if not ws.getSnapshot(nom_glob_name):
-            self.stdout.info('INFO: Saving snapshot "{}"'.format(nom_glob_name))
+            self.stdout.info(f'INFO: Saving snapshot "{nom_glob_name}"')
             ws.saveSnapshot(nom_glob_name, all_globs)
         if not ws.getSnapshot(nom_nuis_name):
-            self.stdout.info('INFO: Saving snapshot "{}"'.format(nom_nuis_name))
+            self.stdout.info(f'INFO: Saving snapshot "{nom_nuis_name}"')
             ws.saveSnapshot(nom_nuis_name, all_nuis)
         
         if do_fit:
@@ -1038,14 +1039,15 @@ class ExtendedModel(AbstractObject):
                 else:
                     # unconditional mle
                     poi.setConstant(0)
-            minimizer.minimize()
-        
+            status = minimizer.minimize()
+            self.last_fit_status = status
+            
         if poi:
             if poi_val is not None:
                 poi.setVal(poi_val)
             poi.setConstant(0)
             poi_val = poi.getVal()
-        
+
         # match values of global observables to the corresponding NPs
         if modify_globs:
             self.match_globs()
@@ -1085,18 +1087,19 @@ class ExtendedModel(AbstractObject):
                                           ROOT.RooFit.Index(channel_cat),
                                           ROOT.RooFit.Import(dataset_map),
                                           ROOT.RooFit.WeightVar(weight_var))
+
         if do_import:
             if ws.data(asimov_data_name):
                 self.stdout.warning(f"WARNING: Dataset with name {asimov_data_name} already exists in the "
                                     "workspace. The newly generated dataset will not overwrite the original "
                                     "dataset.")
             getattr(ws, "import")(asimov_data)
-            self.stdout.info(f'INFO: Generated Asimov Dataset \"{asimov_data_name}\"')
+            self.stdout.info(f'INFO: Generated Asimov Dataset "{asimov_data_name}"')
             
         if asimov_snapshot is not None:
             snapshot_name = asimov_snapshot.format(mu=pretty_value(poi_val),
                                                    mu_cond=pretty_value(poi_profile))
-            self.stdout.info('INFO: Saving snapshot "{}"'.format(snapshot_name))
+            self.stdout.info(f'INFO: Saving snapshot "{snapshot_name}"')
             ws.saveSnapshot(snapshot_name, mutable_vars)
     
         if restore_states == 0:
@@ -1108,7 +1111,7 @@ class ExtendedModel(AbstractObject):
         elif restore_states == 2:
             pass
         else:
-            raise ValueError(f"unsupported restore state option \"{restore_states}\"")
+            raise ValueError(f'unsupported restore state option "{restore_states}"')
         
         return asimov_data
     
@@ -1158,13 +1161,50 @@ class ExtendedModel(AbstractObject):
         
         return toys
     
-    def save(self, fname:str, recreate:bool=True):
+    def save(self, filename:str, recreate:bool=True, rebuild:bool=True,
+             keep_snapshots:Optional[List[str]]=None,
+             keep_datasets:Optional[List[str]]=None):
+        """Save the current workspace as a ROOT file.
+        
+        Parameters:
+        ---------------------------------------------
+        filename: str
+            Name of the output ROOT file.
+        recreate: bool, default = True
+            Recreate the output file if exists.
+        rebuild: bool, default = True
+            Rebuild the workspace from scratch.
+        keep_snapshots: (optional) list of str
+            Snapshots to keep. If not specified, all snapshots will be kept.
+        keep_datasets: (optional) list of str
+            Datasets to keep. If not specified, all datasets will be kept.
         """
+        
         # buggy, model config is messed up
-        new_ws = ROOT.RooWorkspace(self.workspace)
-        new_ws.writeToFile(fname, recreate)
-        """
-        self.workspace.writeToFile(fname, recreate)
+        # new_ws = ROOT.RooWorkspace(self.workspace)
+        # new_ws.writeToFile(filename, recreate)
+
+        # rebuild the entire workspace
+        if rebuild:
+            from quickstats.components.workspaces import XMLWSModifier
+            config = {"data_name": None}
+            if keep_snapshots is not None:
+                config["snapshot_list"] = keep_snapshots
+            else:
+                if (quickstats.root_version >= (6, 26, 0)):
+                    config["snapshot_list"] = [i.GetName() for i in self.workspace.getSnapshots()]
+                else:
+                    self.stdout.warning("WARNING: Saving of snapshots not supported with ROOT version < 6.26.0. "
+                                        "No snapshots will be saved in the rebuilt workspace.")
+                    config["snapshot_list"] = []
+            if keep_datasets is not None:
+                config["dataset_list"] = keep_datasets
+            modifier = XMLWSModifier(config, verbosity="WARNING")
+            modifier.create_modified_workspace(self.workspace, filename,
+                                               import_class_code=False,
+                                               recreate=recreate)
+        else:
+            self.workspace.writeToFile(filename, recreate)
     
     @semistaticmethod
     def load_ws(self, fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
@@ -1718,3 +1758,76 @@ class ExtendedModel(AbstractObject):
         if save_as is not None:
             with open(save_as, "w") as f:
                 f.write(summary_str)
+                
+    def get_systematics_variations(self, filter_name:Optional[str]=None,
+                                   filter_client:Optional[str]=None, fmt:str="pandas"):
+        from quickstats.utils.roofit_utils import get_systematics_variations
+        constr_pdfs, constr_nuis, _ = self.pair_constraints()
+        return get_systematics_variations(constr_nuis, constr_pdfs, filter_name=filter_name,
+                                          filter_client=filter_client, fmt=fmt)
+    
+    def compare_datasets(self, ds1:Union[str, ROOT.RooDataSet], ds2:Union[str, ROOT.RooDataSet]):
+        datasets = {}
+        for label, ds in [("left", ds1), ("right", ds2)]:
+            if isinstance(ds, str):
+                if not self.workspace.data(ds):
+                    raise ValueError(f'workspace does not contain a dataset named "{ds}"')
+                ds = self.workspace.data(ds)
+            if not isinstance(ds, ROOT.RooDataSet):
+                raise TypeError('dataset must be an instance of RooDataSet')
+            datasets[label] = ds
+        dataset_values = {}
+        from quickstats.interface.root import RooDataSet
+        for key, ds in datasets.items():
+            dataset_values[key] = RooDataSet.to_category_data(ds, self.pdf)
+        categories = list(dataset_values["left"])
+        difference = {}
+        identical_cats = []
+        modified_cats = []
+        for category in categories:
+            difference[category] = dataset_values["left"][category] - dataset_values["right"][category]
+            if np.allclose(dataset_values["left"][category], dataset_values["right"][category], rtol=1e-8):
+                identical_cats.append(category)
+            else:
+                modified_cats.append(category)
+        for cat_type, cats in [("identical", identical_cats), ("modified", modified_cats)]:
+            self.stdout.info(f"Categories with {cat_type} dataset values:")
+            for cat in cats:
+                self.stdout.info(f"\t{cat}")
+        return difference
+    
+    def compare_snapshots(self, ss1:Union[str, ROOT.RooArgSet], ss2:Union[str, ROOT.RooArgSet]):
+        snapshots = {}
+        for label, ss in [("left", ss1), ("right", ss2)]:
+            if isinstance(ss, str):
+                if not self.workspace.getSnapshot(ss):
+                    raise ValueError(f'workspace does not contain a snapshot named "{ss}"')
+                ss = self.workspace.getSnapshot(ss)
+            if not isinstance(ss, ROOT.RooArgSet):
+                raise TypeError('dataset must be an instance of RooDataSet')
+            snapshots[label] = ss
+        from quickstats.components.workspaces import ComparisonData, WSItemType
+        from quickstats.interface.root.roofit_extension import get_str_data
+        data = ComparisonData("Snapshots", True, False)
+        content = WSItemType.VARIABLE.default_content
+        style   = WSItemType.VARIABLE.default_style
+        data_1 = get_str_data(snapshots["left"], fill_classes=False,
+                              fill_definitions=True,
+                              content=content, style=style, fmt="dataframe")
+        data_2 = get_str_data(snapshots["right"], fill_classes=False,
+                              fill_definitions=True,
+                              content=content, style=style, fmt="dataframe")
+        data.add(data_1, data_2)
+        data.process()
+        summary_str = data.get_summary_str(visibility=0b01011, indent="    ")
+        self.stdout.info(summary_str)
+    
+    def get_gaussian_constraint_attributes(self, fmt:str="pandas"):
+        constr_pdfs, _, _ = self.pair_constraints()
+        from quickstats.utils.roofit_utils import get_gaussian_pdf_attributes
+        return get_gaussian_pdf_attributes(constr_pdfs, fmt=fmt)
+   
+    def get_poisson_constraint_attributes(self, fmt:str="pandas"):
+        constr_pdfs, _, _ = self.pair_constraints()
+        from quickstats.utils.roofit_utils import get_poisson_pdf_attributes
+        return get_poisson_pdf_attributes(constr_pdfs, fmt=fmt)
\ No newline at end of file
