@@ -8,7 +8,7 @@ from functools import partial
 import numpy as np
 
 import quickstats
-from quickstats import semistaticmethod, AbstractObject, GeneralEnum
+from quickstats import semistaticmethod, AbstractObject, GeneralEnum, PathManager
 from quickstats.utils.io import Verbosity
 from quickstats.utils.xml_tools import TXMLTree
 from quickstats.maths.numerics import to_bool
@@ -69,6 +69,7 @@ class XMLWSBase(AbstractObject):
             self.basedir = os.path.dirname(source)
         else:
             self.basedir = os.getcwd()
+        self.path_manager = PathManager(self.basedir)
         if unlimited_stack:
             set_unlimited_stacksize()
             self.stdout.info("INFO: Set stack size to unlimited")
@@ -81,7 +82,10 @@ class XMLWSBase(AbstractObject):
             result = load_macro(extension)
             if (result is not None) and hasattr(ROOT, extension):
                 self.stdout.info(f'INFO: Loaded extension module "{extension}"')
-            cls = get_macro_TClass(extension)
+            try:
+                cls = get_macro_TClass(extension)
+            except:
+                continue
             self.custom_classes[extension] = cls
     
     @staticmethod
@@ -99,15 +103,7 @@ class XMLWSBase(AbstractObject):
         return os.path.join(self.basedir, filename)
     
     def check_file_exist(self, filename:str):
-        return os.path.exists(self.get_relpath(filename))        
-    
-    @semistaticmethod
-    def _xml_to_dict(self, filename:str):
-        try:
-            xml_dict = TXMLTree(file=filename).to_dict()
-        except:
-            raise RuntimeError(f"failed to load xml document `{filename}`")
-        return xml_dict
+        return os.path.exists(self.get_relpath(filename))
     
     @staticmethod
     def _get_node_attrib(node:Dict, attrib_name:Union[str, List[str]], required:bool=True,
@@ -233,6 +229,21 @@ class XMLWSBase(AbstractObject):
             object_names.append(object_name)
         return ",".join(object_names)
     
+    def import_class_code(self, ws:ROOT.RooWorkspace):
+        all_function_class = [i.ClassName().split("::")[-1] for i in ws.allFunctions()]
+        all_pdf_class = [i.ClassName().split("::")[-1] for i in ws.allPdfs()]
+        all_class = all_function_class + all_pdf_class
+        for extension, cls in self.custom_classes.items():
+            # the workspace does not use the extended class, no need to import
+            if extension not in all_class:
+                continue
+            macro_dir = get_macro_dir(extension)
+            ws.addClassDeclImportDir(macro_dir)
+            ws.addClassImplImportDir(macro_dir)
+            ws.importClassCode(cls)
+            self.stdout.info(f"INFO: Imported class code for \"{cls.GetName()}\".")
+        #ws.importClassCode()    
+    
     def _generate_asimov(self, ws:ROOT.RooWorkspace, asimov_definitions:List, data_name:str,
                          range_name:Optional[str]=None, minimizer_config:Optional[Dict]=None,
                          title_indent_str:str="\t"):
