@@ -11,6 +11,12 @@ CPP_MACROS = \
         std::vector<T> as_vector(const T* data, size_t size) {
             return std::vector<T>(data, data + size);
         }
+        template <typename T>
+        void* as_pointer(std::vector<T> data) {
+             T* result = new T[data.size()];
+             std::copy(data.begin(), data.end(), result);
+             return result;
+         }
     };
     """
 }
