@@ -20,6 +20,14 @@ def to_bool(val:Any):
                 return False
             else:
                 raise ValueError(f"invalid boolean expression: {val}")
+                
+def to_string(val:Any, precision:int=8) -> str:
+    if isinstance(val, float):
+        val = round(val, precision)
+    return str(val)
+
+def to_rounded_float(val:Any, precision:int=8) -> float:
+    return round(float(val), precision)
 
 def pretty_value(val:Union[int, float], precision:int=8)->Union[int, float]:
     if isinstance(val, float):
