@@ -113,6 +113,8 @@ def get_counting_significance(s:float, b:float, sigma_b:float=0, leading_order:b
             sigma_b: float, default = 0
                 Background uncertainty. A zero value means the number of
                 background events is exactly known.
+            leading_order: bool, default=False
+                Whether to use leading order approximation.
     """
     if sigma_b == 0:
         if leading_order:
@@ -143,8 +145,8 @@ def get_combined_counting_significance(s:np.ndarray, b:np.ndarray,
             sigma_b: float / np.ndarray of float, default = 0
                 Array of background uncertainties in each experiment. A zero value
                 means the number of background events is exactly known.
-            axis: int
-                Axis along which the significance is evaluated.
+            leading_order: bool, default=False
+                Whether to use leading order approximation.
     """
     if sigma_b == 0:
         if leading_order:
@@ -212,7 +214,7 @@ def get_hist_data(x:np.ndarray, weights:Optional[np.ndarray]=None,
             pois_interval = get_poisson_interval(_y)
             yerr =  (pois_interval["lo"] / size, pois_interval["hi"] / size)
             _weights = weights / size
-            y, bin_edges = np.histogram(x, bins=bins, range=brange, weights=_weights)
+            y, bin_edges = np.histogram(x, bins=bins, range=range, weights=_weights)
         elif error_method == "sumw2":
             _y, bin_edges = np.histogram(x, bins=bins, range=range, weights=weights**2)
             _weights = weights / size
