@@ -98,6 +98,17 @@ class AbstractPlot:
                 }
             else:
                 raise RuntimeError(f"the handle {handle} does not have an associated label")
+                
+    def add_legend_decoration(self, decorator, targets:List[str]):
+        for key, legend_data in self.legend_data.items():
+            if key not in targets:
+                continue
+            handle = legend_data["handle"]
+            if isinstance(handle, tuple):
+                new_handle = (*handle, decorator)
+            else:
+                new_handle = (handle, decorator)
+            legend_data["handle"] = new_handle
 
     def get_legend_handles_labels(self, sec:bool=False,
                                   idx:Optional[Union[int, List[int]]]=None):
@@ -149,11 +160,11 @@ class AbstractPlot:
     def draw_axis_labels(self, ax, xlabel:Optional[str]=None, ylabel:Optional[str]=None,
                          xlabellinebreak:Optional[int]=None, ylabellinebreak:Optional[int]=None):
         if xlabel is not None:
-            if xlabellinebreak is not None:
+            if (xlabellinebreak is not None) and (xlabel.count("$") < 2):
                 xlabel = insert_periodic_substr(xlabel, xlabellinebreak)
             ax.set_xlabel(xlabel, **self.styles['xlabel'])
         if ylabel is not None:
-            if ylabellinebreak is not None:
+            if (ylabellinebreak is not None) and (ylabel.count("$") < 2):
                 ylabel = insert_periodic_substr(ylabel, ylabellinebreak)            
             ax.set_ylabel(ylabel, **self.styles['ylabel'])
             
