@@ -1,7 +1,7 @@
 from typing import Optional, Union, Dict, List, Tuple, Callable
+import numbers
 
 from cycler import cycler
-
 import numpy as np
 
 from quickstats.plots.color_schemes import QUICKSTATS_PALETTES
@@ -500,12 +500,7 @@ class PdfDistributionPlot(AbstractPlot):
                          n_bins_data:Optional[int]=None,
                          n_bins_pdf:Optional[int]=1000,
                          blind_range:Optional[List[float]]=None,
-                         label_map:Optional[Dict]=None,
-                         color_cycle:Optional[Dict]=None,
-                         styles:Optional[Union[Dict, str]]=None,
-                         analysis_label_options:Optional[Union[Dict, str]]=None,
-                         figure_index:Optional[int]=None,
-                         config:Optional[Dict]=None):
+                         **kwargs):
         from quickstats.utils.roofit_utils import export_hist_data
         hist_data = export_hist_data(data, pdf,
                                      observable=observable,
@@ -513,8 +508,32 @@ class PdfDistributionPlot(AbstractPlot):
                                      bin_error=bin_error,
                                      n_bins_data=n_bins_data,
                                      n_bins_pdf=n_bins_pdf)
-        plotter = cls(hist_data, blind_range=blind_range,
-                      label_map=label_map, color_cycle=color_cycle,
-                      styles=styles, analysis_label_options=analysis_label_options,
-                      figure_index=figure_index, config=config)
-        return plotter
\ No newline at end of file
+        plotter = cls(hist_data, **kwargs)
+        return plotter
+    
+    @classmethod
+    def from_dataframes(cls, collective_df:Dict[str, "pandas.DataFrame"],
+                        observable:str, weight:Optional[Union[float, str]]=None,
+                        range:Optional[Union[List, Tuple]]=None,
+                        bins:int=10, normalize:bool=True,
+                        x_scale:Optional[float]=None,                        
+                        error_method:str="auto", **kwargs):
+        from quickstats.maths.statistics import get_hist_data
+        collective_data = {}
+        for key, df in collective_df.items():
+            x = df[observable].values 
+            if x_scale is not None:
+                x = x * x_scale
+            if weight is None:
+                weights = None
+            elif isinstance(weight, numbers.Number):
+                weights = np.full(shape=len(df.index), fill_value=weight)
+            else:
+                weights = df[weight].values
+            hist_data = get_hist_data(x, weights, normalize=normalize,
+                                      range=range, bins=bins, error_method=error_method)
+            collective_data[key] = hist_data
+        plotter = cls(collective_data, **kwargs)
+        return plotter
+        
+                        
\ No newline at end of file
