@@ -17,7 +17,7 @@ class UpperLimit2DPlot(AbstractPlot):
             "linewidth": 1,
             "markersize": 5,
             "marker": 'o',
-        }        
+        }
     }
     
     COLOR_PALLETE = {
@@ -27,7 +27,7 @@ class UpperLimit2DPlot(AbstractPlot):
         'observed': 'k'
     }
     
-    COLOR_PALLETE_SEC = {
+    COLOR_PALLETE_EXTRA = {
         '2sigma': '#FDC536',
         '1sigma': '#4AD9D9',
         'expected': 'r',
@@ -41,7 +41,7 @@ class UpperLimit2DPlot(AbstractPlot):
         'observed': 'Observed limit (95% CL)'
     }
     
-    LABELS_SEC = {
+    LABELS_EXTRA = {
         '2sigma': 'Expected limit $\pm 2\sigma$',
         '1sigma': 'Expected limit $\pm 1\sigma$',
         'expected': 'Expected limit (95% CL)',
@@ -65,30 +65,44 @@ class UpperLimit2DPlot(AbstractPlot):
             'markersize' : 20,
             'color' : '#E9F1DF',
             'markeredgecolor' : 'black'
+        },
+        'errorband_plot_styles':{
+            'alpha': 1
+        },
+        'expected_plot_styles': {
+            'marker': 'None',
+            'linestyle': '--',
+            'alpha': 1,
+            'linewidth': 1
+        },
+        'observed_plot_styles': {
+            'marker': 'o',
+            'alpha': 1,
+            'linewidth': 1
         }
     }
     
-    def __init__(self, data, data_sec=None,
-                 scale_factor=None,
+    def __init__(self, data:pd.DataFrame,
+                 additional_data:Optional[List[Dict]]=None,
+                 scale_factor:float=None,
                  color_pallete:Optional[Dict]=None,
-                 color_pallete_sec:Optional[Dict]=None,
                  labels:Optional[Dict]=None,
-                 labels_sec:Optional[Dict]=None,
                  styles:Optional[Union[Dict, str]]=None,
                  analysis_label_options:Optional[Union[Dict, str]]='default',
                  config:Optional[Dict]=None):
         super().__init__(color_pallete=color_pallete,
-                         color_pallete_sec=color_pallete_sec,
                          styles=styles,
                          analysis_label_options=analysis_label_options,
                          config=config)
         self.data     = data
-        # secondary data
-        self.data_sec = data_sec
+        
+        self.additional_data = []
+        if additional_data is not None:
+            for _data in additional_data:
+                self.add_data(**_data)
         
         self.labels = combine_dict(self.LABELS, labels)
-        self.labels_sec = combine_dict(self.LABELS_SEC, labels_sec)
-            
+ 
         self.scale_factor = scale_factor
         
         self.curve_data     = None
@@ -137,26 +151,23 @@ class UpperLimit2DPlot(AbstractPlot):
             handle_fill = ax.fill_between(data['x'], data['yerrlo'], data['yerrhi'],
                                           label=data['label'], **fill_styles)
             handles = (handle_fill, handle_line[0])
-        self.update_legend_handles({'curve': handles})
+        self.update_legend_handles({'curve': handles}, idx=0)
         
     def draw_highlight(self, ax, data):
         styles = data['styles']
         if styles is None:
             styles = self.config['highlight_styles']
         handle = ax.plot(data['x'], data['y'], label=data['label'], **styles)
-        self.update_legend_handles({'highlight': handle[0]})
+        self.update_legend_handles({'highlight': handle[0]}, idx=0)
         
     def draw_single_data(self, ax, data, scale_factor=None,
                          log:bool=False, 
                          draw_observed:bool=True,
                          color_pallete:Optional[Dict]=None,
                          labels:Optional[Dict]=None,
-                         observed_marker:Optional[str]='o', 
                          sigma_band_hatch:Optional[str]=None,
-                         alpha:float=1.,
-                         linewidth:float=1.,
-                         sec:bool=False,
-                         draw_errorband:bool=True):
+                         draw_errorband:bool=True,
+                         idx:int=0):
         
         if color_pallete is None:
             color_pallete = self.color_pallete
@@ -180,12 +191,12 @@ class UpperLimit2DPlot(AbstractPlot):
                                             facecolor=color_pallete['2sigma'],
                                             label=labels['2sigma'],
                                             hatch=sigma_band_hatch,
-                                            alpha=alpha)
+                                            **self.config["errorband_plot_styles"])
             handle_1sigma = ax.fill_between(indices, n1sigma_limits, p1sigma_limits, 
                                             facecolor=color_pallete['1sigma'],
                                             label=labels['1sigma'],
                                             hatch=sigma_band_hatch,
-                                            alpha=alpha)
+                                            **self.config["errorband_plot_styles"])
             handles_map['1sigma'] = handle_1sigma
             handles_map['2sigma'] = handle_2sigma
         
@@ -198,43 +209,51 @@ class UpperLimit2DPlot(AbstractPlot):
             obs_limits = data['obs'].values * scale_factor
             handle_observed = draw_fn(indices, obs_limits, color=color_pallete['observed'], 
                                       label=labels['observed'], 
-                                      marker=observed_marker,
-                                      alpha=alpha, linewidth=linewidth)
+                                      **self.config["observed_plot_styles"])
             handles_map['observed'] = handle_observed[0]
             
         handle_expected = draw_fn(indices, exp_limits, color=color_pallete['expected'],
-                                  linestyle='--',
                                   label=labels['expected'],
-                                  alpha=alpha, linewidth=linewidth)
+                                  **self.config["expected_plot_styles"])
         handles_map['expected'] = handle_expected[0]
 
-        self.update_legend_handles(handles_map, sec=sec)
+        self.update_legend_handles(handles_map, idx=idx)
+        
+    def add_data(self, data:pd.DataFrame, color_pallete:Optional[Dict]=None,
+                 labels:Optional[Dict]=None, draw_observed:bool=False,
+                 draw_errorband:bool=False):
+        config = {
+            "data": data,
+            "color_pallete": combine_dict(self.COLOR_PALLETE_EXTRA, color_pallete),
+            "labels": combine_dict(self.LABELS_EXTRA, labels),
+            "draw_observed": draw_observed,
+            "draw_errorband": draw_errorband
+        }
+        self.additional_data.append(config)
             
     def draw(self, xlabel:str="", ylabel:str="", ylim=None, xlim=None,
              log:bool=False, draw_observed:bool=True, draw_errorband:bool=True,
-             draw_sec_errorband:bool=False, observed_marker:Optional[str]='o',
-             draw_hatch:bool=True, linewidth:float=1):
+             draw_sec_errorband:bool=False, draw_hatch:bool=True):
         
         ax = self.draw_frame()
         
-        if self.data_sec is not None:
+        if len(self.additional_data) > 0:
             if draw_hatch:
                 sigma_band_hatch = self.config['secondary_hatch']
                 alpha = self.config['secondary_alpha']
             else:
                 sigma_band_hatch = None
                 alpha = 1.
-            self.draw_single_data(ax, self.data_sec,
-                                  scale_factor=self.scale_factor,
-                                  log=log, 
-                                  draw_observed=draw_observed,
-                                  color_pallete=self.color_pallete_sec,
-                                  labels=self.labels_sec,
-                                  observed_marker=observed_marker, 
-                                  sigma_band_hatch=sigma_band_hatch,
-                                  alpha=alpha,
-                                  sec=True,
-                                  draw_errorband=draw_sec_errorband)
+            for idx, config in enumerate(self.additional_data):
+                self.draw_single_data(ax, config["data"],
+                                      scale_factor=self.scale_factor,
+                                      log=log, 
+                                      draw_observed=config["draw_observed"],
+                                      color_pallete=config["color_pallete"],
+                                      labels=config["labels"],
+                                      sigma_band_hatch=sigma_band_hatch,
+                                      draw_errorband=config["draw_errorband"],
+                                      idx=idx + 1)
             if draw_hatch:
                 sigma_band_hatch = self.config['primary_hatch']
                 alpha = self.config['primary_alpha']
@@ -250,11 +269,10 @@ class UpperLimit2DPlot(AbstractPlot):
                               draw_observed=draw_observed,
                               color_pallete=self.color_pallete,
                               labels=self.labels,
-                              observed_marker=observed_marker, 
                               sigma_band_hatch=sigma_band_hatch,
-                              alpha=alpha,
-                              linewidth=linewidth,
-                              draw_errorband=draw_errorband)
+                              draw_errorband=draw_errorband,
+                              idx=0)
+        
         if self.curve_data is not None:
             self.draw_curve(ax, self.curve_data)   
         if self.highlight_data is not None:
@@ -269,23 +287,16 @@ class UpperLimit2DPlot(AbstractPlot):
 
         # border for the legend
         border_leg = patches.Rectangle((0, 0), 1, 1, facecolor = 'none', edgecolor = 'black', linewidth = 1)
-        
-        if draw_errorband:
-            self.legend_data['1sigma']['handle'] = (self.legend_data['1sigma']['handle'], border_leg)
-            self.legend_data['2sigma']['handle'] = (self.legend_data['2sigma']['handle'], border_leg)
+        for legend_data in self.legend_data_ext.values():
+            for sigma in ['1sigma', '2sigma']:
+                if sigma in legend_data:
+                    legend_data[sigma]['handle'] = (legend_data[sigma]['handle'], border_leg)        
         
         if self.curve_data is not None:
-            if isinstance(self.legend_data['curve']['handle'], tuple):
-                self.legend_data['curve']['handle'] = (*self.legend_data['curve']['handle'], border_leg)
+            if isinstance(self.legend_data_ext[0]['curve']['handle'], tuple):
+                self.legend_data_ext[0]['curve']['handle'] = (*self.legend_data_ext[0]['curve']['handle'], border_leg)
         
-        if (self.data_sec is not None) and (draw_sec_errorband):
-            self.legend_data_sec['1sigma']['handle'] = (self.legend_data_sec['1sigma']['handle'], border_leg)
-            self.legend_data_sec['2sigma']['handle'] = (self.legend_data_sec['2sigma']['handle'], border_leg)
-            
-        handles, labels = self.get_legend_handles_labels()
-        if self.data_sec is not None:
-            handles_sec, labels_sec = self.get_legend_handles_labels(sec=True)
-            handles = handles + handles_sec
-            labels  = labels + labels_sec
+        indices = sorted(self.legend_data_ext.keys())
+        handles, labels = self.get_legend_handles_labels(idx=indices)
         ax.legend(handles, labels, **self.styles['legend'])
         return ax
