@@ -1,6 +1,8 @@
 from typing import Optional
 import re
 
+whitespace_trans = str.maketrans('', '', " \t\r\n\v")
+
 def split_lines(string:str, comment_string:Optional[str]="#", remove_blank:bool=True,
                 with_line_number:bool=False):
     lines = string.splitlines(True)
@@ -24,4 +26,7 @@ def split_str(string:str, strip:bool=True, remove_empty:bool=False):
     if remove_empty:
         # remove empty tokens
         tokens = [s for s in tokens if s]
-    return tokens
\ No newline at end of file
+    return tokens
+
+def remove_whitespace(string:str):
+    return string.translate(whitespace_trans)
\ No newline at end of file
