@@ -8,7 +8,12 @@ class TXMLElement(ET.Element):
         super().__init__(tag, attrib, **extra)
         self.text = text
         self.tail = tail
-        
+    
+    def get_attribute(self, name:str):
+        if name not in self.attrib:
+            raise ValueError(f'XML element "{self.tag}" does not contain the attribute "{name}"')
+        return self.attrib[name]
+    
     @classmethod
     def from_dict(cls, source):
         if not isinstance(source, dict):
@@ -165,6 +170,11 @@ class TXMLElement(ET.Element):
     
 
 class TXMLTree(ET.ElementTree):
+    
+    @property
+    def root(self):
+        return self._root
+    
     def __init__(self, element=None, file=None,
                  version="1.0", encoding="UTF-8", doctype=None, system=None):
         super().__init__(element, file)
@@ -272,7 +282,15 @@ class TXMLTree(ET.ElementTree):
     def to_dict(self):
         if self._root is None:
             return {}
-        return self._root.to_dict()     
+        return self._root.to_dict()
+    
+    @staticmethod
+    def load_as_dict(filename:str):
+        try:
+            xml_dict = TXMLTree(file=filename).to_dict()
+        except:
+            raise RuntimeError(f'failed to load xml document "{filename}"')
+        return xml_dict
     
     def to_diagram(self, indent=' '*4, show_attrib=False):
         if self._root is None:
